#pragma once

#include "BuddyUser.h"

class CMessengerActor;

class CBuddyGroup : public IGTBuddyGroup
{
public:
	CBuddyGroup(CMessengerActor *pMessengerActor);
	virtual ~CBuddyGroup(void);

	// For GTObject
protected:	
	virtual void OnReleaseObject(void) { delete this; }
	
public:	
	virtual int GetID(void) const
	{
		return m_sData.nID;
	}
	virtual int GetSortOrder(void) const
	{
		return m_sData.nSortOrder;
	}
	virtual LPCWSTR GetName(void) const
	{
		return m_sData.strName.c_str();
	}

	virtual void GetUpdatableData(GT_BUDDY_GROUP_UPDATABLE_DATA &sData) const
	{
		m_sData.GetUpdatableData(sData);
	}
	
	virtual bool IsDefaultGroup(void) const;

	void SetData(const GT_BUDDY_GROUP_DATA &userdata)
	{
		m_sData.Load(userdata);
	}
	void SetData(const P_GT_BUDDY_GROUP_DATA &userdata)
	{
		m_sData = userdata;
	}
	void SetSortID(int nSortNum)
	{
		m_sData.nSortOrder = nSortNum;
	}

	void LoadData(const GT_BUDDY_GROUP_UPDATABLE_DATA &userdata);
	void LoadData(const P_GT_BUDDY_GROUP_UPDATABLE_DATA &userdata);
	
	inline const P_GT_BUDDY_GROUP_DATA & GetData()
	{
		return m_sData;
	}

	inline void GetData(GT_BUDDY_GROUP_DATA &result)
	{
		m_sData.Convert(result);
	}
	
	size_t GetBuddyCount(void)
	{
		return m_vecBuddy.size();
	}

	bool AddBuddyUser(CBuddyUser *pBuddyUser)
	{
        std::pair<buddymap_t::iterator, bool> ret = m_mapBuddy.insert(buddymap_t::value_type(pBuddyUser->GetUserID(), pBuddyUser));
                                                                      
        if(ret.second == false)
        {
            // Already inserted..
            return false;
        }
        
		buddyvector_t::iterator it = std::find(m_vecBuddy.begin(), m_vecBuddy.end(), pBuddyUser);
		if(it == m_vecBuddy.end())
		{
			m_vecBuddy.push_back(pBuddyUser);
		}
        
        return true;
	}

	void DelBuddyUser(CBuddyUser *pBuddyUser)
	{
        buddymap_t::iterator itMap = m_mapBuddy.find(pBuddyUser->GetUserID());
        
        if(itMap != m_mapBuddy.end())
        {
            m_mapBuddy.erase(itMap);
        }

		buddyvector_t::iterator it = std::find(m_vecBuddy.begin(), m_vecBuddy.end(), pBuddyUser);
		if(it != m_vecBuddy.end())
		{
			m_vecBuddy.erase(it);
		}        
	}
	
	void CleanBuddyUserList(void)
	{
		m_vecBuddy.clear();
        m_mapBuddy.clear();
	}

	CBuddyUser* GetBuddyInfoFromIndex(size_t nIndex)
	{
		if(nIndex >= m_vecBuddy.size())
			return NULL;
		return m_vecBuddy[nIndex];
	}
	
	void ReleseGroup()
	{
		ReleaseObject();
	}

private:
	P_GT_BUDDY_GROUP_DATA m_sData;
	buddyvector_t m_vecBuddy;
	buddymap_t m_mapBuddy;  // For performance..
	CMessengerActor *m_pMessengerActor;
};

typedef std::vector<CBuddyGroup *> buddygroupvector_t;
